// @ myth - client - news.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "client/news.h"
#include "common/error.h"

// @@ const

// Size of news window.
static const int NewsSize = 1;
static const int NewsPerCent = 15;

// @@ NEWS

// @@@ void NEWS::display(int line)

void NEWS::display(int line)
{
    current = line;
    unread = max(unread, min(current + size().y, page.length()));

    page.reset(line);

    for(int y = 0; y < size().y; y++)
    {
	if(page.string() == NULL)
	    print(coor(0, y), "");
	else
	{
	    print(coor(0, y), page.string());
	    page.next();
	}
    }
    touched = FALSE;
}

// @@@ void NEWS::insert(STRING message)

void NEWS::insert(STRING message)
{
    page.append(message);

    if(current + size().y >= page.length())
	touched = TRUE;
}

// @@@ void NEWS::insert(const char* message)

void NEWS::insert(const char* message)
{ insert(STRING(message)); }

// @@@ void NEWS::up(int arg)

void NEWS::up(int arg)
{
    touched = TRUE;

    if(arg != 0)
	current -= arg;
    else if(size().y > 2)
	current -= size().y - 1;
    else
	current -= size().y;

    if(current < 0)
	current = 0;
}    

// @@@ void NEWS::down(int arg)

void NEWS::down(int arg)
{
    touched = TRUE;

    if(arg != 0)
	current += arg;
    else if(size().y > 2)
	current += size().y - 1;
    else
	current += size().y;

    if(current > unread)
	current = unread;
}

// @@@ void NEWS::read()

void NEWS::read()
{
    if(current != unread)
    {
	current = unread;
	touched = TRUE;
    }
}

// @@@ void NEWS::update(force flag)

void NEWS::update(force flag)
{
    if(touched)
	display(current);
    else if(flag == FORCE &&
	    unread >= current + size().y &&
	    unread < page.length())
	display(unread);
}

// @@@ NEWS::NEWS()

NEWS::NEWS() : (WINDOW_BOTTOM, NewsSize, NewsPerCent)
{
    unread = 0;
    current = 0;
    touched = FALSE;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
