// @ myth - client - prompt.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "client/prompt.h"
#include "common/config.h"
#include "common/error.h"
#include "common/parse.h"
#include "common/page.h"
#include "common/format.h"

// @@ const

// Smallest size of window, for which argument will be included in the prompt
static const ArgumentIncludeSize = 25;
// The prompt window is two lines heigh
static const PromptWindowSize = 2;

// @@ SELECT

// @@@ class SELECT

class SELECT
{
    PAGE data[PROMPT_END];
public:
    STRING find(int width, prompt_type prompt, long arg, STRING value);
    SELECT(STRING name);
};

// @@@ STRING SELECT::find(int width,

STRING SELECT::find(int width, prompt_type prompt, long arg, STRING value)
{
    PAGE& page = data[prompt];

    page.reset();
    
    while(page.remain() > 0)
    {
	STRING string;
	
	if((width < ArgumentIncludeSize || arg == 0))
	    string = format(page.string(), value());
	else
	    string = format("(%ld) %s", arg,
			    format(page.string(), value()).string());

	if(string.length() < width)
	    return string;

	page.next();
    }
    return NullString;
}

// @@@ SELECT::SELECT(STRING name)

SELECT::SELECT(STRING name)
{
    PARSE parse(name);

    if(parse.LookAhead(INPUT_COMMENT))
	parse.skip(INPUT_COMMENT);

    while(parse.good())
    {
	prompt_type type = PromptToType(parse.GetWord());
	parse.skip(INPUT_COMMENT);

	while(parse.good() && parse.LookAhead(">"))
	{
	    parse.skip(">");
	    data[type].append(parse.GetLine());
	}
	if(parse.LookAhead(INPUT_COMMENT))
	    parse.skip(INPUT_COMMENT);
    }	
    if(!parse.eof())
	bug.error("SELECT::SELECT", format("failed to parse '%s'", name()));
}

// @@@ SELECT selection

static SELECT selection(format("%s/%s/%s",
			       HomeDirectory, LibraryDirectory, "PROMPT"));

// @@  PROMPT

// @@@ void PROMPT::select(prompt_type one,

void PROMPT::select(prompt_type one, const char* message,
		    prompt_type two, long arg, const char* value)
{
    STRING val(value);
    
    if(one != first || message != error)
    {
	first = one;
	error = message;
	FirstTouched = TRUE;
    }
    if(two != second || arg != argument || val != string)
    {
	second = two;
	argument = arg;
	string = val;
	SecondTouched = TRUE;
    }
}

// @@@ void PROMPT::update(force flag = CHECK)

void PROMPT::update(force flag = CHECK)
{
    static int length = 0;

    if(FirstTouched)
    {
	if(error == NULL)
	    print(Origo, selection.find(size().x, first, 0L, NullString));
	else
	    print(Origo, error);

	FirstTouched = FALSE;
    }
    if(SecondTouched)
    {
	STRING value = selection.find(size().x, second, argument, string);
	print(coor(0, 1), value);
	length = value.length();
	SecondTouched = FALSE;
    }
    if(flag == FORCE)
	cursor(coor(length, 1));
}

// @@@ PROMPT::PROMPT()

PROMPT::PROMPT() : (WINDOW_TOP, PromptWindowSize)
{
    first = second = PROMPT_IDLE;
    argument = 0L;
    error = NULL;
    FirstTouched = SecondTouched = TRUE;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
