// @ myth - server - fight.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/level.h"
#include "server/player.h"
#include "server/rnd.h"
#include "server/object.h"
#include "server/weapon.h"
#include "server/ray.h"
#include "common/format.h"

// @@ void LEVEL::fight(MONSTER* attack, MONSTER* defence)

void LEVEL::fight(MONSTER* attack, MONSTER* defence)
{
    attack->UseFightPoint(75 + 2 * defence->experience());
    attack->UseMovePoint(150 + 4 * defence->experience());

    (void) hit(defence->position(),
	       attack->strength() * 2 + attack->ToDamage(),
	       attack->weapon(), attack, NoWhere);
}

// @@ void LEVEL::throw(MONSTER* monster, OBJECT* object, coor target)

void LEVEL::throw(MONSTER* monster, OBJECT* object, coor target)
{
    monster->UseFightPoint(50);
    monster->UseMovePoint(50);

    int power = 0;		// Power object is thrown with
    OBJECT* weapon = monster->weapon();

    // Tell the world
    monster->message(format("You throw %s.", ::print(object, PRINT_OBJECT)));
    WatchMonster(monster, "%s throws %s.", object);

    // If any parameters is silly, just drop the object
    if(target != Origo && object->weight() > 0L && object->size() > 0)
    {
	// Calculate the power 
	power += object->ThrowBonus(monster, weapon);
	power += monster->strength() + monster->experience();
	power -= MyLog(object->weight());
	power -= MyLog(object->size() / object->weight());
	power = max(0, power);

	// Adjust the target (He don't throw that precise)
	target *= max(1, 500 / max(abs(target.x), abs(target.y)));
	int bonus = 50 - monster->experience() - monster->dexterity();
	if(monster->IsConfused())
	    bonus *= 10;
	
	if(bonus > 0)
	{
	    target.x += rnd.chance(2) ? -rnd(bonus) : rnd(bonus);
	    target.y += rnd.chance(2) ? -rnd(bonus) : rnd(bonus);
	}
    }
    straight line(monster->position(), monster->position() + target);

    for(;;)
    {
	if(power < 1)
	{
	    BUNCH* bunch = create(line);
	    WatchPosition(line, format("%s falls down on the ground.",
				       print(object, PRINT_NAME |
					             PRINT_COUNT |
					             PRINT_UPPER)));
	    bunch->add(object);
	    return;
	}
	coor last(line);
	line.next();

	if(hit(line, power, object, monster, last))
	    return;
	
	power--;
    }
}

// @@ void LEVEL::zap(MONSTER* monster, RAY* ray, coor target)

void LEVEL::zap(MONSTER* monster, RAY* ray, coor target)
{
    // Adjust the target (He don't zap that precise)
    if(target != Origo)
    {
	target *= max(1, 500 / max(abs(target.x), abs(target.y)));
	int bonus = 26 - monster->experience();
	if(monster->IsConfused())
	    bonus *= 10;
	
	if(bonus > 0)
	{
	    target.x += rnd.chance(2) ? -rnd(bonus) : rnd(bonus);
	    target.y += rnd.chance(2) ? -rnd(bonus) : rnd(bonus);
	}
    }
    if(target == Origo)		// Zap yourself!
	(void) ray->zap(this, monster, target);
    else
    {
	straight line(monster->position(), monster->position() + target);
	line.next();

	while(ray->zap(this, monster, line) == CONTINUE)
	    line.next();
    }
    delete ray;
}

// @@ void LEVEL::dig(MONSTER* monster, coor pos)

void LEVEL::dig(MONSTER* monster, coor pos)
{
    monster->UseMovePoint(250);
    monster->UseFightPoint(5);
    
    (void) hit(pos, monster->strength() * 3,
	       monster->weapon(), monster, NoWhere);
}

// @@ bool LEVEL::hit(coor pos, int power,

bool LEVEL::hit(coor pos, int power,
		OBJECT* object, MONSTER* monster, coor last)
{
    bool IsThrown = (last != NoWhere);
    BUNCH* bunch = LEVEL::bunch(pos);
    OBJECT* target = (bunch == NULL) ? NULL : bunch->HitOne();
    STRING done;
    const char* verbum = "hits";
    STRING ObjectName(print(object, PRINT_NAME));
    STRING OwnerName(format("%s's",
			    print(monster, PRINT_MONSTER | PRINT_UPPER)));
    STRING OtherName;
    bool DoHit = TRUE;
    
    if(target != NULL)
    {
	OtherName = PrintObject(target, PRINT_MONSTER);

	if(IsThrown && target->catch(power, object))
	    verbum = "is catched by";
	else if(target->hit(power, object, monster))
	{
	    done = target->hurt(power, object, monster, this);
	    if(IsThrown)
	    {
		bunch = create(pos);
		bunch->add(object);
	    }
	}
	else
	{
	    verbum = "misses";
	    DoHit = FALSE;
	}
    }
    else
    {
	int rock = LEVEL::rock(pos);

	// Passes straight through
	if(!IsRock(rock))
	    return FALSE;

	OtherName = STRING("the wall", STRING_STATIC);

	// Damage the weapon
	if(rnd.chance(power,
		      power + object->strength() * 5 +
		      object->DigBonus(monster) * 5))
	{
	    if(object->IsBodyPart())
	    {
		done = monster->hurt(power, object, monster, this);
		verbum = "is hurt against";
	    }
	    else
	    {
		object->damage();
		if(!IsThrown)
		    monster->pack.touch();
	    }
	}
	// Damage the wall
	else if(rnd.chance(power + object->DigBonus(monster),
			   RockStrength(rock)))
	{
	    dig(pos, rock_type(rock - 1));
	    if(!IsRock(rock - 1))
	    {
		done = STRING("The dungeon wall crumble.", STRING_STATIC);
		bunch = create(pos);
		bunch->add(new STONE(rnd(10, 100)));
	    }
	}
	// Drop the missile
	if(IsThrown)
	{
	    bunch = create(pos);
	    
	    if(rnd.chance(5))
		bunch->add(object);
	    else
		bunch->add(object);
	}
    }
    coor MonsterPosition = monster->position();
    PLAYER* player;
    const char* other;
    const char* owner;

    party.reset();
    while((player = party.current()) != NULL)
    {
	if(player->CanSee(pos))
	{
	    if(player == target)
		other = (player == monster) ? "yourself" : "you";
	    else
		other = OtherName();
	    
	    if(player == monster ||
	       player->CanSee(MonsterPosition))
	    {
		owner = (player == monster) ? "Your" : OwnerName();
		player->message(format("%s %s %s %s.",
				       owner, ObjectName(), verbum, other));
	    }
	    else
		player->message(format("Some %s %s %s.",
				       ObjectName(), verbum, other));

	    if(done.length() > 0)
	    {
		if(target == NULL)
		{
		    if(player == monster)
			player->message(format(done(), "You"));
		    else
			player->message(format(done(), OwnerName()));
		}
		else if(target == player)
		    player->message(format(done(), "You"));
		else
		    player->message(format(done(), OtherName()));
	    }
	}
	party.next();
    }
    return DoHit;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
