// @ myth - server - id.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/id.h"
#include "common/binary.h"
#include "server/object.h"
#include "server/rnd.h"
#include "server/invent.h"

// @@ IDENTITY

// @@@ void IDENTITY::load(BINARY* binary)

void IDENTITY::load(BINARY* binary)
{ id = LoadIdentity(binary); }

// @@@ void IDENTITY::save(BINARY* binary)

void IDENTITY::save(BINARY* binary)
{ SaveIdentity(binary, id); }

// @@@ IDENTITY identity

IDENTITY identity;

// @@ id_type LoadIdentity(BINARY* binary)

id_type LoadIdentity(BINARY* binary)
{ return binary->GetLong(); }

// @@ void SaveIdentity(BINARY* binary, id_type id)

void SaveIdentity(BINARY* binary, id_type id)
{ binary->PutLong(id); }

// @@ ID_LIST

// @@@ void ID_LIST::insert(id_type id)

void ID_LIST::insert(id_type id)
{
    id_type* new_id = new id_type;
    (*new_id) = id;
    list.add(new_id);
}

// @@@ void ID_LIST::erase()

void ID_LIST::erase()
{
    delete ((id_type*) list.current());
    list.remove();
}

// @@@ bool ID_LIST::member(OBJECT* object)

bool ID_LIST::member(OBJECT* object)
{
    id_type id = object->identity();
    
    reset();
    while(!done())
    {
        if(id == current())
	    return TRUE;
        else
	    next();
    }
    return FALSE;
}

// @@@ void ID_LIST::remove(OBJECT* object)

void ID_LIST::remove(OBJECT* object)
{ 
    id_type id = object->identity();
    
    reset();
    while(!done())
    {
        if(id == current())
	    erase();
        else
	    next();
    }
}

// @@@ void ID_LIST::add(OBJECT* object)

void ID_LIST::add(OBJECT* object)
{
    if(!member(object))
        insert(object->identity());
}

// @@@ void ID_LIST::forget(int chance, INVENTORY* inventory)
//
// Remove one out CHANCE entry, which doesn't apear in INVENTORY
// To be used when players change level, to remove old entries

void ID_LIST::forget(int chance, INVENTORY* inventory)
{
    reset();
    while(!done())
    {
        if(rnd.chance(chance))
        {
	    SPECIFY_IDENTITY specify(current());
	    
	    if(inventory->contain(&specify) == FALSE)
		erase();
	    else
		next();
        }
        else
	    next();
    }
}

// @@@ void ID_LIST::forget(int chance)
//
// Remove one out CHANCE entry.

void ID_LIST::forget(int chance)
{
    reset();
    while(!done())
    {
        if(rnd.chance(chance))
	    erase();
        else
	    next();
    }
}

// @@@ void ID_LIST::save(BINARY* binary)

void ID_LIST::save(BINARY* binary)
{
    reset();
    while(!done())
    {
        SaveIdentity(binary, current());
        next();
    }
    SaveIdentity(binary, NoIdentity);
}

// @@@ void ID_LIST::load(BINARY* binary)

void ID_LIST::load(BINARY* binary)
{
    id_type id = LoadIdentity(binary);
    
    while(id != NoIdentity)
    {
        insert(id);
        id = LoadIdentity(binary);
    }
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
