// @ myth - server - potion.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/potion.h"
#include "server/monster.h"
#include "server/level.h"
#include "server/ability.h"
#include "common/error.h"
#include "server/rnd.h"

// @@ POTION_STRENGTH

// @@@ void POTION_STRENGTH::effect(LEVEL*, MONSTER* monster)

void POTION_STRENGTH::effect(LEVEL*, MONSTER* monster)
{
    int add = rnd(rnd(MyValue + 1) + 1) + 1;

    if(IsCursed())
    {
	monster->message("Yech! This stuff tastes like poison.");
	monster->AddStrength(-add);
	monster->RemoveAbility(ABILITY_SICK, FORCE);
    }
    else if(IsBlessed())
    {
	monster->message("Wow do you feel strong!");
	monster->AddStrength(add);
    }
    else
    {
	monster->message("Wow! This makes you feel great!");
	monster->RestoreStrength();
    }	
    monster->learn(this);
}

// @@ POTION_HEALING

// @@@ void POTION_HEALING::effect(LEVEL*, MONSTER* monster)

void POTION_HEALING::effect(LEVEL*, MONSTER* monster)
{
    if(IsCursed())
    {
	monster->message("Huh? What was that!");
	monster->AddAbility(new COUNTER_CONFUSED(rnd(MyValue * 40) + MyValue));
    }
    else if(IsBlessed())
    {
	monster->message("You feel much better.");
	monster->RestoreHealth();
	monster->RemoveAbility(ABILITY_CONFUSED, CHECK);
	monster->RemoveAbility(ABILITY_INFECTED, FORCE);
	if(monster->CheckAbility(ABILITY_SICK))
	    monster->AddAbility(new COUNTER_SICK(MyValue * MyValue));
    }
    else
    {
	monster->message("You feel better.");
	monster->heal(rnd(rnd(MyValue + 1) + 1) + 1);
	monster->RemoveAbility(ABILITY_BLIND, CHECK);
	monster->RemoveAbility(ABILITY_INFECTED, CHECK);
    }
    monster->learn(this);
}

// @@ POTION_EXPERIENCE

// @@@ void POTION_EXPERIENCE::effect(LEVEL*, MONSTER* monster)

void POTION_EXPERIENCE::effect(LEVEL*, MONSTER* monster)
{
    int add = rnd(rnd(MyValue * monster->experience() + 1) + 1) + 1;

    if(IsCursed())
    {
	monster->message("You feel stupid.");
	monster->GainExperience(-add);
    }
    else if(IsBlessed())
    {
	monster->message("You feel lucky.");
	monster->GainExperience(add);
	monster->GainScore(add);
	monster->RemoveAbility(ABILITY_CONFUSED, FORCE);
    }
    else
    {
	monster->message("You feel clever.");
	monster->GainExperience(add);
	monster->RemoveAbility(ABILITY_CONFUSED, CHECK);
    }
    monster->learn(this);
}

// @@ POTION_SIGHT

// @@@ void POTION_SIGHT::effect(LEVEL*, MONSTER* monster)

void POTION_SIGHT::effect(LEVEL*, MONSTER* monster)
{
    int count = rnd(MyValue * 20) + MyValue;

    if(IsCursed())
    {
	monster->message("A cloud of darkness falls upon you.");
	monster->AddAbility(new COUNTER_BLIND(count));
	monster->learn(this);
    }
    else
    {
	int what = rnd(3);
	
	if(what == 0 || IsBlessed())
	    monster->AddAbility(new COUNTER_SEE_INVISIBLE(count));
	if(what == 1 || IsBlessed())
	{
	    monster->message("You sence the precence of monsters.");
	    monster->AddAbility(new COUNTER_DETECT(count));
	    monster->learn(this);
	}
	if(what == 2 || IsBlessed())
	{
	    monster->message("You feel your vision improve.");
	    monster->AddAbility(new COUNTER_SIGHT(count));
	    monster->learn(this);
	}
	monster->RemoveAbility(ABILITY_BLIND, IsBlessed() ? FORCE : CHECK);
    }
}

// @@ POTION_ENERGY

// @@@ void POTION_ENERGY::effect(LEVEL* level, MONSTER* monster)

void POTION_ENERGY::effect(LEVEL* level, MONSTER* monster)
{
    int add = rnd(MyValue * MyValue * MyValue);

    if(IsCursed())
    {
	monster->sleep(level, add);

	if(!monster->sleeping())
	{
	    monster->message("The world speed up.");
	    monster->AddAbility(new COUNTER_SLOW(rnd(MyValue * MyValue) +
						 MyValue));
	}
	monster->RemoveAbility(ABILITY_FAST, CHECK);
	monster->learn(this);
    }
    else if(IsBlessed())
    {
	monster->message("The world slow down.");
	monster->AddAbility(new COUNTER_FAST(rnd(MyValue * 25) + MyValue));
	monster->RemoveAbility(ABILITY_SLOW, FORCE);
	monster->learn(this);
    }
    else
    {
	monster->sleep(level, add);

	if(monster->CheckAbility(ABILITY_SLOW))
	{
	    monster->RemoveAbility(ABILITY_SLOW, CHECK);
	    monster->learn(this);
	}
	else if(add > 60)
	    monster->learn(this);
    }
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
